// This is a -*- C++ -*- header.
//========================================================================
// last updated : [2008/02/04] -17:21:38(JST)-
//  written by $Author: okamura $ on $Date: 2008/09/01 09:55:49 $.
// -----------------------------------------------------------------------
// $Name:  $-$Revision: 4.1 $-$State: Exp $
// $Id: ne.ccqe.cc,v 4.1 2008/09/01 09:55:49 okamura Exp $
//========================================================================
// -----------------------------------------------------------------------
//  COMMENT
// ---------
//  fit function for the CCQE event for nu_e
//-----------------------------------------------------------------------
#include <cmath>
//! definition of \f$  \sqrt{2\pi}  \f$ for easy calculate
#define SQRT2PI   (2.5066282746310005024157652848110452530069867406099)

//---------------------------
//! \short calculate the energy shift : \f$ \delta E_1^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta E_1^e (E_\nu) \f$ (MeV)
//---------------------------
double E1necc(double E0)
{
      return 42.9806
          -0.0097203*E0
          +5.43518e-6 *E0*E0
          -7.13136e-10*E0*E0*E0
          -2185.2/E0;
}

//---------------------------
//! \short calculate the deviation : \f$ \sigma_1^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \sigma_1^e (E_\nu) \f$ (MeV)
//---------------------------
double s1necc(double E0)
{
    return -23.8594+3.12737*sqrt(E0)-0.0195692*E0;
}

//---------------------------
//! \short calculate the energy shift : \f$ \delta E_2^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta E_2^e (E_\nu) \f$ (MeV)
//---------------------------
double E2necc(double E0)
{
    double E2=E1necc(E0);
    return E2;
}

//---------------------------
//! \short calculate the deviation : \f$ \sigma_2^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \sigma_2^e (E_\nu) \f$ (MeV)
//---------------------------
double s2necc(double E0)
{
    return -25.0091 + 6.37138*sqrt(E0) - 0.0513354*E0;
}

//---------------------------
//! \short calculate the ratio : \f$ r_2^e (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ r_2^e (E_\nu) \f$
//---------------------------
double r2necc(double E0)
{
    return 2.01884
        -0.00231993 *E0
        +1.19516e-6 *E0*E0
        -2.42188e-10*E0*E0*E0
        +1.93105e-14*E0*E0*E0*E0;
}

//---------------------------
//! \short calculate the deviation : \f$ \sigma_3^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \sigma_3^e (E_\nu) \f$ (MeV)
//---------------------------
double s3necc(double E0)
{
    return -571.589
        +45.7657*sqrt(E0)
        -0.71303*E0
        +0.00349619*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : \f$ \delta E_3^e (E_\nu) \f$ (MeV) 
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta E_3^e (E_\nu) \f$ (MeV)
//---------------------------
double E3necc(double E0)
{
    double sr3=s3necc(E0);
    double er3=(2.82283-0.0809032*sqrt(E0)
                +0.000679759*E0-1082.1/E0);
    return sr3*er3;
}

//---------------------------
//! \short calculate the  ratio : \f$ r_3^e (E_\nu) \f$ 
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ r_3^e (E_\nu) \f$
//---------------------------
double r3necc(double E0)
{
    return 0.185905 
        - 0.000172685*E0
        + 1.13859e-07*E0*E0
        - 3.95206e-11*E0*E0*E0
        + 5.94879e-15*E0*E0*E0*E0;
}

//---------------------------
//! \short calculate the normalization factor : \f$ A^e (E_\nu) \f$ 
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return normalization factor : \f$ A^e (E_\nu) \f$
//---------------------------
double Anecc(double E0)
{
    double s1=s1necc(E0);

    double s2=s2necc(E0);
    double r2=r2necc(E0);

    double s3=s3necc(E0);
    double r3=r3necc(E0);

    return SQRT2PI*(s1 + s2*r2 + s3*r3);
}

/*!
 \short calculate the \f$ f_e^{\rm CCQE}(E_{\rm rec};E_\nu) \f$
 \param  En  : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
 \param  ene : reconstructed     energy, \f$ E_{\rm rec} \f$ (MeV)
 \return probability of the event distribution
 \f$ f_e^{\rm CCQE}(E_{\rm rec};E_\nu) \f$
*/

double Fnecc(double En,double ene)
{
    double E1=En-E1necc(En);
    double E2=En-E2necc(En);
    double E3=En-E3necc(En);

    double s1=s1necc(En);
    double s2=s2necc(En);
    double s3=s3necc(En);

    double r2=r2necc(En);
    double r3=r3necc(En);

    double a=Anecc(En);

    double fitfc = 
        1.0/a*(1.0*exp(-((ene-E1)*(ene-E1))/(2.0*s1*s1))
               +r2*exp(-((ene-E2)*(ene-E2))/(2.0*s2*s2))
               +r3*exp(-((ene-E3)*(ene-E3))/(2.0*s3*s3)));

    return fitfc;

}
