// This is a -*- C++ -*- header.
//========================================================================
// last updated : [2009/01/11] -00:29:21(JST)-
//  written by $Author: okamura $ on $Date: 2008/09/01 09:55:49 $.
// -----------------------------------------------------------------------
// $Name:  $-$Revision: 4.1 $-$State: Exp $
// $Id: nm.ccqe.cc,v 4.1 2008/09/01 09:55:49 okamura Exp $
//========================================================================
// -----------------------------------------------------------------------
//  COMMENT
// ---------
//  fit function for the CCQE event for nu_mu 
//-----------------------------------------------------------------------
#include <cmath>
#include <iostream>
#include <iomanip>

//! definition of \f$  \sqrt{2\pi}  \f$ for easy calculate
#define SQRT2PI   (2.5066282746310005024157652848110452530069867406099)

//---------------------------
//! \short calculate the energy shift : \f$ \delta E_1^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta E_1^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E1nmcc(double E0)
{
    return 27.204 + 5353.17/E0+0.00286368*E0;
}

//---------------------------
//! \short calculate the deviation : \f$ \sigma_1^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \sigma_1^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s1nmcc(double E0)
{
    return 2.16623*sqrt(E0)-28.806;
}

//---------------------------
//! \short calculate the energy shift : \f$ \delta E_2^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta E_2^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E2nmcc(double E0)
{
    double E2=E1nmcc(E0);
    return E2;
}

//---------------------------
//! \short calculate the deviation : \f$ \sigma_2^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \sigma_2^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s2nmcc(double E0)
{
//    return -150.536 - 0.0581716*E0 + 9.15206*sqrt(E0);
    return -126.537 - 0.0501716*E0 + 8.20339*sqrt(E0);
}

//---------------------------
//! \short calculate the ratio : \f$ r_2^\mu (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ r_2^\mu (E_\nu) \f$
//---------------------------
double r2nmcc(double E0)
{
    return 2.59235
        -0.00208821 *E0
        +6.95013e-7 *E0*E0
        -9.50673e-11*E0*E0*E0
        +4.73664e-15*E0*E0*E0*E0;
}

//---------------------------
//! \short calculate the deviation : \f$ \sigma_3^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \sigma_3^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s3nmcc(double E0)
{
    return 113.796
        -12.7377*sqrt(E0)
        +0.688661*E0 
        -0.00641136*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : \f$ \delta E_3^\mu (E_\nu) \f$ (MeV) 
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta E_3^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E3nmcc(double E0)
{
    double s3=s3nmcc(E0);
    double e3=(36335.9-59.6058*E0)/(E0*E0-1162.09*E0+391785)
        + 0.24477 
        -0.000104744*E0 
        +1.38022e-08*E0*E0;
    return s3*e3;
}

//---------------------------
//! \short calculate the  ratio : \f$ r_3^\mu (E_\nu) \f$ 
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ r_3^\mu (E_\nu) \f$
//---------------------------
double r3nmcc(double E0)
{
    return -5.30548
        +0.213334*sqrt(E0)
        -0.00322444*E0
        +1.74138e-05*E0*sqrt(E0)
        +1302.78/(E0-184.927);
//    return exp(1717.39/(E0+125.988)
//               -0.710517
//               -0.00216082*E0
//               +3.46692e-7*E0*E0);
}

//---------------------------
//! \short calculate the normalization factor : \f$ A^\mu (E_\nu) \f$ 
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return normalization factor : \f$ A^\mu (E_\nu) \f$
//---------------------------
double Anmcc(double E0)
{
    double s1=s1nmcc(E0);

    double s2=s2nmcc(E0);
    double r2=r2nmcc(E0);

    double s3=s3nmcc(E0);
    double r3=r3nmcc(E0);

    return SQRT2PI*(s1 + s2*r2 + s3*r3);
}

/*!
 \short calculate the \f$ f_\mu^{\rm CCQE}(E_{\rm rec};E_\nu) \f$
 \param  En  : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
 \param  ene : reconstructed     energy, \f$ E_{\rm rec} \f$ (MeV)
 \return probability of the event distribution
 \f$ f_\mu^{\rm CCQE}(E_{\rm rec};E_\nu) \f$
*/

double Fnmcc(double En,double ene)
{
    double E1=En-E1nmcc(En);
    double E2=En-E2nmcc(En);
    double E3=En-E3nmcc(En);

    double s1=s1nmcc(En);
    double s2=s2nmcc(En);
    double s3=s3nmcc(En);

    double r2=r2nmcc(En);
    double r3=r3nmcc(En);

    double a=Anmcc(En);

    double fitfc = 
        1.0/a*(1.0*exp(-((ene-E1)*(ene-E1))/(2.0*s1*s1))
               +r2*exp(-((ene-E2)*(ene-E2))/(2.0*s2*s2))
               +r3*exp(-((ene-E3)*(ene-E3))/(2.0*s3*s3)));

    return fitfc;

}
